<?php // Include only
if (!defined ('BOOT')) exit ('No bootstrap.  Exiting.');

/*
OekakiPoteto 5.x Copyright 2000-2002 RanmaGuy (Theo Chakkapark, http://suteki.nu) and Marcello Basta-Forte (http://marcello.cellosoft.com). Modification to the files are permitted as long as my name remains on the files.

Wax Poteto modifications Copyright Marc "Waccoon" Leveille
http://www.NineChime.com/products/
Version 5.8.10 - Last modified 11/26/2012
*/


// ===================================================================
// Misc
//
// Microtime
function get_microtime() {
	$mtime = microtime();
	$mtime = explode (' ', $mtime);
	return doubleval ($mtime[1]) + doubleval ($mtime[0]);
}


function diff_microtime($start_time) {
	return (substr(get_microtime() - $start_time, 0, 5));
}


// ===================================================================
// GPC
//

// Wrapper for HTML encoding.
function w_html_chars($in) {
	return (htmlspecialchars ($in));
}

function w_toalpha ($in) {
	// Note underscore
	return (preg_replace ("/[^a-zA-Z0-9\s_]/", '', $in));
}

// Get GPC value and filter it
// Complex, because PHP arrays and magic quotes are a PITA
// All methods are slashed except 'raw'
// filter (''=>escaped, 'raw'=>plain, 'a'=>alphanumeric, 'i'=>int, 'i+'=>positive_int', 'html'=>html_encoded, 'url'=>url, 'email'=>email)
function w_gpc ($key, $filter = '') {
	$value = FALSE;

	// Get GPC in order of importance
	// There is no significant cross of GET/POST/COOKIE names
	if (isset ($_GET[$key]))
		$value = $_GET[$key];
	if (isset ($_POST[$key]))
		$value = $_POST[$key];
	if (isset ($_COOKIE[$key]))
		$value = $_COOKIE[$key];

	// Some web objects (like checkboxes) will not be set if value is blank.  Make sure ints are returned properly!
	if ($value === FALSE) {
		if ($filter == 'i') {
			return 0;
		} else {
			return (NULL);
		}
	}

	// Severe filter (alphanumeric, w/underscore)
	if ($filter == 'a') {
		$value = is_array ($value)
			? array_map ('w_toalpha', $value)
			: w_toalpha ($value);

		return ($value);
	}

	// Handle magic quotes
	if (get_magic_quotes_gpc()) {
		// Remove slashes for raw
		if ($filter == 'raw') {
			$value = is_array ($value)
				? array_map ('stripslashes', $value)
				: stripslashes ($value);
		}
	} else {
		// Add slashes for all except raw and i
		if ($filter != 'raw' && $filter != 'i' && $filter != 'i+') {
			$value = is_array ($value)
				? array_map ('addslashes', $value)
				: addslashes ($value);
		}
	}


	// Filter
	$op = '';
	$plus = false;
	switch ($filter) {
		case 'email':
			$op = 'w_validate_email';
			break;
		case 'html':
			$op = 'w_html_chars';
			break;
		case 'i':
			$op = 'intval';
			break;
		case 'i+':
			$op = 'intval';
			$plus = true;
			break;
		case 'raw':
			break;
		case 'url':
			$op = 'w_validate_url';
			break;
		default:
			$op = $filter;
			break;
	}
	if ($filter == 'html') {
		// Remove slashes only from double quotes
		if (is_array ($value)) {
			for ($i = 0; $i < count ($value); $i++) {
				$value[$i] = str_replace ('\\"', '"', $value[$i]);
			}
		} else {
			$value = str_replace ('\\"', '"', $value);
		}
	}

	// Main filter
	if (!empty ($op)) {
		$value = (is_array ($value))
			? array_map ($op, $value)
			: $op ($value);
	}

	// Positive ints only?
	if ($plus) {
		if (is_array ($value)) {
			for ($i = 0; $i < count ($value); $i++) {
				$value[$i] = ($value[$i] < 0) ? 0 : $value[$i];
			}
		} else {
			$value = ($value < 0) ? 0 : $value;
		}
	}

	return ($value);
}


function w_set_cookie($name, $value, $exp = FALSE) {
	// Always call after hacks/cfg
	global $cookie_life, $cookie_path, $cookie_domain;

	if ($exp === FALSE)
		$exp = time() + $cookie_life;

	$result = setcookie ($name, $value, $exp, $cookie_path, $cookie_domain);
	return $result;
}


function w_unset_cookie($name) {
	// Always call after hacks/cfg
	global $cookie_path, $cookie_domain;

	$result = setcookie ($name, '', mktime (12, 0, 0, 1, 1, 1970), $cookie_path, $cookie_domain);
	return $result;
}


// ===================================================================
// Error reporting
//

// Blank location sends to index.php
// "${loc};in" logs in, "${loc};out" logs out
// Presense of semicolon dictates redirect, so "in" and "out" are not only tags
function all_done($location = 'index.php') {
	$url = $location;

	// Hack to fix IIS cookie issue
	if (strpos ($location, ';') !== FALSE) {
		list ($url, $stat) = explode (';', $location);

		if (strpos (strtoupper ($_SERVER['SERVER_SOFTWARE']), 'IIS/') !== FALSE) {
			if ($stat == 'in') {
				$mess = 'Thanks for logging in!';
			} elseif ($stat == 'out') {
				$mess = 'You have sucessfully logged out.';
			} else {
				$mess = 'Your login has been updated.';
			}

			echo <<<EOF
<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
<head>
	<meta http-equiv="Refresh" content="3;url={$url}">
	<title>Redirect</title>
</head>
<body>
	<p style="text-align: center; padding: 15px; border: 2px black solid;">
		<strong>{$mess}</strong><br /><br />If you are not redirected in 3 seconds, click <a href="{$url}">here</a>.
	</p>
</body>
</html>
EOF;
			w_exit();
		}
	}

	header('Location: '.$url);
	w_exit();
}


function report_err($error = '', $small_header = FALSE) {
	global $langop_common;

	if (empty ($error)) {
		$error = $langop_common['error'];
	}
	if ($small_header) {
		$small_header = '&s=1';
	}

	header ('Location: error.php?error='.urlencode($error).$small_header);
	w_exit();
}


function w_exit($message = '') {
	db_close();

	if (!empty($message)) {
		/* Test if plain text
		if (strstr('<', $message) === FALSE || strstr('&', $message)) {
		}
		*/
		echo ($message);
	}

	if (isset($GLOBALS['zlib']) && $GLOBALS['zlib']) {
		ob_end_flush();
	}

	exit();
}


/*
	w_log() error logging
	Instigator is always $OekakiU (or cron/system/guest)
	Usage(NOTICE):
		w_log(WLOG_CONST|$string, $text, $affected);
	Usage(STOP):
		w_log(WLOG_CONST|$string, "{$file}:__LINE__: {$text}", $affected);
*/
define('WLOG_MISC', 'misc');
define('WLOG_FAIL', 'fail'); // All errors not related to SQL
define('WLOG_SQL_FAIL', 'sqlfail'); // All SQL errors
define('WLOG_MAINT', 'maintenance'); // Self/cron jobs
define('WLOG_SECURITY', 'security'); // Permission/security/filtering issues
define('WLOG_BANNER', 'banner');
define('WLOG_RULES', 'rules');
define('WLOG_NOTICE', 'notice');
define('WLOG_CPANEL', 'cpanel');
define('WLOG_THUMB_OVERRIDE', 'thumbchange');
define('WLOG_THUMB_REBUILD', 'thumbrebuild');
define('WLOG_MASS_MAIL', 'massmail');
define('WLOG_BAN', 'ban');
define('WLOG_DELETE_USER', 'kill');
define('WLOG_REG', 'registration');
define('WLOG_APPROVE', 'approve');
define('WLOG_EDIT_PROFILE', 'editprofile');
define('WLOG_FLAGS', 'flags');
define('WLOG_PASS_RECOVER', 'passrecover');
define('WLOG_PASS_RESET', 'passreset');
define('WLOG_ARCHIVE', 'archive');
define('WLOG_BUMP', 'bump');
define('WLOG_RECOVER', 'recover');
define('WLOG_DELETE', 'delete');
define('WLOG_LOCK_THREAD', 'lockthread');
define('WLOG_ADULT', 'adult');
define('WLOG_ADMIN_WIP', 'adminwip');
define('WLOG_EDIT_PIC', 'editpicture');
define('WLOG_EDIT_COMM', 'editcomment');
function w_log ($category, $value, $affected = '') {
	global $dbconn, $db_p, $db_mem;
	global $OekakiU;
	global $cfg;
	global $address; // IP address

	$log_max_items = 500;

	// Trim log if needed
	$result = db_query ("SELECT COUNT(ID) FROM {$db_mem}oekakilog");
	if (intval (db_result ($result, 0)) > $log_max_items) {
		$result = db_query ("SELECT ID FROM {$db_mem}oekakilog ORDER BY ID LIMIT 1");
		if ($result) {
			$log_del = intval (db_result ($result, 0)) + 4;
			db_query ("DELETE FROM {$db_mem}oekakilog WHERE ID < $log_del");
		}
	}

	// Fix category if needed
	if (empty ($category)) {
		$category = WLOG_MISC;
	}

	// Determine instigator
	if ($category == WLOG_MAINT) {
		$member = '(System)';
	} else {
		$member = '(IP: '.$address.')';
		if (!empty($OekakiU)) {
			$member = addslashes ($OekakiU);
		}
	}

	// Add to log
	$board    = addslashes ($cfg['op_title']);
	$category = addslashes ($category);
	$value    = addslashes ($value);
	$affected = addslashes ($affected);

	$query = "INSERT INTO {$db_mem}oekakilog SET category='{$category}', value='{$value}', member='{$member}', affected='{$affected}', board='{$board}', date=NOW()";
	$result = db_query ($query);

	if ($result) {
		return TRUE;
	}
	return FALSE;
}


/*
	w_log() wrapper, used for SQL messages
	Usage:
		w_log_sql('{$file}:'.__LINE__.": ".$text, $member_optional);
		w_log_sql('{$file}: '.$text, $member_optional);
*/
function w_log_sql($value, $affected = '') {
	global $OekakiU;

	if (empty ($affected)) {
		$affected = $OekakiU;
	}

	return (w_log (WLOG_SQL_FAIL, $value.': ('.db_error().')', $affected));
}


// ===================================================================
// Untainting
//
function badChars($text, $type = 'db') {
	// Code from PHP.net

	if ($type == 'db')
		$illegal = array ('\'', '"', '$', '#', '&', '+', '=', '*', '/', '\\', ':', ';', '<', '>');
	if ($type == 'file')
		$illegal = array ('\'', '"', '$', '#', '?', '=', '*', '/', '\\', ':', '|', '<', '>');

	$garb = array();

	foreach ($illegal as $k) {
		while (($j = strpos ($text, $k)) !== FALSE) {
			$garb[] = $text{$j};
			$text = substr ($text, 0, $j).substr ($text, $j + 1);
		}
	}
	$garb = array_unique ($garb);

	return $garb;
}


function w_validate_email($in) {
	// Returns strings only, so use with db calls or w_gpc()
	// Addresses only!  Does not support "mailto:" protocol (which is a URL)

	// IMPORTANT: We must trim \n from addresses for PHP's mail() call,
	// or else there is a spam zombie vulnerability (may allow "cc:").
	// The regex takes care of white space
	$in = trim ($in);

	$pattern = <<<EOF
/^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/
EOF;

	if (preg_match($pattern, $in)) {
		return ($in);
	} else {
		return (NULL);
	}
}


function w_validate_url($in) {
	// Returns strings only, so use with db calls or w_gpc()

	// Just for the heck of it
	$in = trim ($in);

	// Regex pattern by Benjamin Gray @ regexlib.com
	// Modified by Marc Leveille for PHP5 and to include AIM protocol
	// $1=full, $2=protocol, $3=host, $9=query w/slash
	// Use PHP's parse_url(), not this function, to split parts
	$pattern = <<<EOF
!^((https?|ftp|aim)\://((\[?(\d{1,3}\.){3}\d{1,3}\]?)|(([-a-zA-Z0-9]+\.)+[a-zA-Z]{2,4}))(\:\d+)?(/[-a-zA-Z0-9._?,'+&amp;%$#=~\\\]+)*/?)$!
EOF;

	if (preg_match($pattern, $in)) {
		return ($in);
	} else {
		return (NULL);
	}
}


// Prepare path input (UNIX/URI only)
function strip_path($in) {
	// Strip trailing slash(es)
	$in = preg_replace ('/\/+$/', '', $in);

	// Strip root, but not backtick
	$in = preg_replace ('/^(\.\/)+/', '', $in);

	return $in;
}


function fix_range($low, $high, $number) {
	// Clip to a given range (no arrays. I'm lazy)
	// Don't confuse with PHP range() function!
	if ($number < $low)
		return $low;
	if ($number > $high)
		return $high;
	return $number;
}


// ===================================================================
// File, stream, PHP wrappers
//
// Checks for empty file
function file_not_empty($in) {
	if (file_exists ($in) && filesize ($in) > 0) {
		return 1;
	} else {
		return 0;
	}
}


// PHP < 4.3 fix
if (!function_exists ('file_get_contents')) {
	function file_get_contents($file) {
		if (file_exists ($file)) {
			$size = @filesize ($file);
			$fp = @fopen ($file, 'rb');
			if ($fp && $size > 0) {
				$contents = fread ($fp, $size);
				fclose ($fp);
				return $contents;
			}
			trigger_error ("file_get_contents('{$file}'): cannot open for reading", E_USER_NOTICE);
		}
		// No data
		return;
	}
}


function file_unlock($in, $force = FALSE) {
	/*
		Wrapper for unlink().  $force bypasses wrapper.

		UNIX:  Helps to prevent permission errors with unwritable files.  These files may usually be deleted, even if they cannot be opened for writing.

		WIN32:  PHP has a problem with its file virtualization layer which causes slow deletes.  Attempting to fopen() a deleted file within a few seconds of deleting it will return a permission denied error.
	*/

	if (file_exists ($in)) {
		$win_kill = FALSE;
		if (strtoupper (substr (PHP_OS, 0, 3)) == 'WIN')
			$win_kill = TRUE;

		if (!$force && $win_kill) {
			// OK, this is Windows, and PHP doesn't like it
			$temp = dirname($in).'/'.'junk_'.rand(0, 32768).'.tmp';

			if (!@rename ($in, $temp)) {
				// This sucks, but is rare
				if (!@unlink ($in)) {
					return FALSE;
				}
			} else {
				if (!@unlink ($temp)) {
					return FALSE;
				}
			}
		} else {
			// UNIX, or brute force
			if (!@unlink ($in)) {
				return FALSE;
			}
		}
	}
	return TRUE;
}


function w_group_readable($file, $new_chmod = '0644') {
	if (defined ('GROUP_READABLE') && GROUP_READABLE) {
		if (file_exists ($file)) {
			if (@chmod ($file, octdec ($new_chmod))) {
				return TRUE;
			}
		}
		return FALSE;
	}
}


function get_filetype($data, $type = '') {
	// Get types in approx. order of popularity
	// If no match, use file ext as type
	// Todo:  detect ASCII corruption (easy with PNG files)

	if (substr ($data, 0, 8) == "\x89PNG\x0D\x0A\x1A\x0A") {
		if (strpos ($data, 'IEND') !== FALSE) {
			// Not truncated
			$type = 'png';
		}
	} elseif (substr ($data, 0, 2) == "\xFF\xD8") {
		// JFIF only.  EXIF is not allowed.
		$type = 'jpg';
	} elseif (substr ($data, 0, 3) == 'GIF') {
		$type = 'gif';
	} elseif (substr ($data, 0, 2) == 'BM') {
		$type = 'bmp';
	} elseif (substr ($data, 0, 23) == "\x00\x00\x00\x0CjP\x20\x20\x0D\x0A\x87\x0A\x00\x00\x00\x14ftypjp2") {
		$type = 'jp2';
	} elseif (substr ($data, 0, 4) == '8BPS') {
		// Note: 8-bit, 16-bit, and HDR images have same header
		$type = 'psd';
	} elseif (substr ($data, 0, 2) == 'PK') {
		$type = 'zip';
	} elseif (substr ($data, 0, 3) == 'FWS') {
		$type = 'swf';
	} elseif (substr ($data, 0, 3) == 'CWS') {
		$type = 'swf';
	}

	return $type;
}


function w_mail ($email, $subject, $message) {
	global $cfg;

	$my_headers = "From: OekakiPoteto <{$cfg['op_email']}>\nX-Mailer: Wacintaki BBS\n";
	$out = @mail ($email, $subject, $message, $my_headers);

	if (DISABLE_EMAIL_CONFIRMATION == 1) {
		return (TRUE);
	}
	return ($out);
}


// ===================================================================
// Encoders/Decoders
//
function w_crypt($in) {
	// Crypt w/out key visible to public
	global $cfg;

	$out = crypt ($in, $cfg['salt']);
	return (substr ($out, 2));
}


function parse_flags($usrflags, $rank = FALSE) {
	$flags = array();
	$flags['all']    = $usrflags;
	$flags['member'] = 0; // Is a registered member
	$flags['G']      = 0; // Is a registered member (old flag)
	$flags['mod']    = 0; // Is a moderator
	$flags['admin']  = 0; // Is an admin
	$flags['sadmin'] = 0; // Is a super admin
	$flags['owner']  = 0; // Is owner
	$flags['rank']   = 0; // (0-9)
	$flags['D']      = 0; // Draw access
	$flags['M']      = 0; // Anim access
	$flags['I']      = 0; // Immunity
	$flags['U']      = 0; // Upload access
	$flags['X']      = 0; // Adult access

	// Assign each flag as a key in $flags array
	$len = strlen ($usrflags);
	for ($i = 0; $i < $len; $i++) {
		$chr = $usrflags{$i};
		$flags[$chr] = $chr;
	}

	// Fix membership
	if ($flags['G']) {
		$flags['member'] = 1;
	}

	// Get admin flags
	$flags['rank'] = intval ($rank);
	if ($flags['rank'] >= RANK_MOD)
		$flags['mod']    = 1;
	if ($flags['rank'] >= RANK_ADMIN) {
		$flags['A'] = 1;
		$flags['admin']  = 1;
	}
	if ($flags['rank'] >= RANK_SADMIN) {
		$flags['S'] = 1;
		$flags['sadmin'] = 1;
	}
	if ($flags['rank'] >= RANK_OWNER) {
		$flags['O'] = 1;
		$flags['owner']  = 1;
	}

	return ($flags); // Keyed array
}


function get_age($in) {
	// Check if an age already.
	// Age has no dashes.
	if (empty ($in)) {
		return '';
	}
	if (strpos ($in, '-') === FALSE) {
		return intval ($in);
	}

	// Age from birthday
	// Thanks to Jean-Marc on the LiquidPulse.net forum
	list ($year, $month, $day) = explode ('-', $in);
	$z = -5; // Timezone

	$z = time() + $z * 3600;
	$yeardiff  = gmdate ('Y', $z) - intval ($year);
	$monthdiff = gmdate ('m', $z) - intval ($month);
	$daydiff   = gmdate ('j', $z) - intval ($day);

	if (($monthdiff < 0) || (($monthdiff == 0) && ($daydiff < 0) )) {
		$age = $yeardiff - 1;
	} else {
		$age = $yeardiff;
	}

	return $age;
}


function decode_birthday($year, $month, $day) {
	$year  = (int) $year;
	$month = (int) $month;
	$day   = (int) $day;

	if ($year < 1900) {
		return '';
	}

	if ($month < 1 || $month > 12) {
		$month = '??';
	} else {
		$month = sprintf ('%02d', $month);
	}
	if ($day < 1 || $day > 31) {
		$day = '??';
	} else {
		$day = sprintf ('%02d', $day);
	}

	return $year.'-'.$month.'-'.$day;
}


function age_to_date($in) {
	if (!isset ($_GLOBALS['datef'])) {
		$datef = array();
		$datef['birthdate'] = 'Y/n/j';
		$datef['birthmonth'] = 'Y/n';
		$datef['birthyear'] = 'Y';
	} else {
		global $datef;
	}

	if (empty ($in)) {
		return '';
	}
	if (strpos ($in, '-') === FALSE) {
		// Old "age" format will not work at all.
		return '';
	}

	list ($year, $month, $day) = explode ('-', $in);
	$my_filter = $datef['birthdate'];
	if ($day == '??') {
		$month = intval ($month);
		$day = 15;
		$my_filter = $datef['birthmonth'];
	}
	if ($month == '??') {
		$month = 6;
		$my_filter = $datef['birthyear'];
	}

	return date ($my_filter, strtotime ($year.'/'.$month.'/'.intval($day)));
}


// Determine best "image_size" param for oekaki applets.
function applet_image_size_param($size_x, $size_y, $ratio = .35) {
	/*
	Input: canvas size (x, y), ratio (optional)
	Output: max filesize, in K

	A lot of testing with actual oekakis showed me that for images larger than 10K, average PNG compression is between .20 to .25, while average JPEG is .12 (low quality) to .25 (medium quality).  It's reasonable to say that if a PNG is getting a worse compression ratio than this, it is the wrong format, and JPEG should be used.  We'll use .35 as the ratio, since the "original" images should still be treated as archive quality.  This is intended to reduce server space usage -- let the thumbnail generator worry about page-view bandwidth.
	*/

	$base_size = 20; // Minimum return, in K
	$area = $size_x * $size_y * 3;
	$max_file_size = round (($area * $ratio) / 1024);

	return ($max_file_size > $base_size) ? $max_file_size : $base_size;
}


// Replace '@' in e-mails
function email_code($in, $clickable = TRUE) {
	// '&middot;' == Win32 '&#149;'
	// '&#064;' == ASCII '@'
	// '&bull' == '&#8226';

	$at_symbol = ($clickable) ? '&#064;' : ' &bull; ';
	return (str_replace ('@', $at_symbol, $in));
}


// ===================================================================
// Web printing functions
//
function p_line($line) {
	echo '<p>'.$line."</p>\n";
}


function web_line($line = '') {
	echo $line."<br />\n";
}


function sql_check($result, $extra = '') {
	if (!$result) {
		echo db_error();
		if (!empty ($extra)) {
			echo ' ['.$extra.']';
		}
		echo "<br />\n";
		return 0;
	}
	return 1;
}


// Returns a string from an array, with seperators and delimiters.
function make_list($array, $sep, $delimiter = '') {
	if (is_scalar ($array))
		return ($delimiter.$array.$delimiter);

	return ($delimiter.implode ($delimiter.$sep.$delimiter, $array).$delimiter);
}


// "Not Empty" Echo.  Prints nothing, or alternate $if_empty, if $string is empty.
function ne_echo($string, $if_empty, $format = '') {
	$string = trim ($string);
	if (!empty ($string)) {
		if (!empty ($format)) {
			echo (str_replace ('{s}', $string, $format));
		} else {
			echo $string;
		}
	} else {
		echo $if_empty;
	}
	return;
}


// Deluxe page numbers
// Takes a keyed array of paramaters ($in) and returns an HTML-formatted block of page numbers
function full_page_numbers($first, $current, $total, $display, $link, $in) {
	/*
	$first = First page number.  Some BBSes use 0 as the first page, others 1.
	$display = Number of pages to show per side of the current page.  0 = use default.

	$in = array:
	prev_enabled  => Linked "Prev", parsed for '{page}'
	prev_disabled => Unlinked "Prev"
	next_enabled  => Linked "Next", parsed for '{page}'
	next_disabled => Unlinked "Next"
	button_sep    => Seperates Prev/Next from numbers
	link_format   => Link format (for classes, etc), parsed for '{page}'
	active_format => Span for current [active] number, parsed for '{page}'
	elipses       => Seperator for first/last numbers and all others
	page_sep      => Seperator between numbers
	*/

	// Fixes offset issue with some BBSes
	function offset_replace($offset, $number, $string) {
		$old = array ('{page_link}', '{page}');
		$new = array ($number + $offset, $number);
		return str_replace ($old, $new, $string);
	}

	// Set up
	$output = '';
	if ($display < 1) {
		$display = 4;
	}
	if ($first == 0) {
		$offset = -1;
		$first += 1;
		$current += 1;
	} else {
		$offset = 0;
	}
	$page_link = str_replace ('{link}', $link, $in['link_format']);
	$in['prev_enabled'] = str_replace ('{link}', $link, $in['prev_enabled']);
	$in['next_enabled'] = str_replace ('{link}', $link, $in['next_enabled']);

	// Get our extremes
	$list_start = 1;
	$list_end   = $total;
	if ($current > $display + 1) {
		$list_start = $current - $display;
	}
	if ($total - $current > $display) {
		$list_end = $current + $display;
	}

	// Print start block
	if ($current > 1) {
		$output .= offset_replace($offset, $current - 1, $in['prev_enabled']);
	} else {
		$output .= $in['prev_disabled'];
	}
	$output .= $in['button_sep'];

	// Print start number?
	if ($list_start > 1) {
		$output .= offset_replace($offset, 1, $page_link);
		$output .= $in['elipses'];
	}

	// Print all numbers
	for ($i = $list_start; $i <= $list_end; $i++) {
		if ($i == $current) {
			$output .= offset_replace($offset, $i, $in['active_format']);
		} else {
			$output .= offset_replace($offset, $i, $page_link);
		}
		if ($i != $list_end) $output .= $in['page_sep'];
	}

	// Print end number?
	if ($list_end < $total) {
		$output .= $in['elipses'];
		$output .= offset_replace($offset, $total, $page_link);
	}

	// Print end block
	$output .= $in['button_sep'];
	if ($current < $total) {
		$output .= offset_replace($offset, $current + 1, $in['next_enabled']);
	} else {
		$output .= $in['next_disabled'];
	}

	return $output;
}


// Simple page numbers (no images)
// Easy, format-free wrapper for full_page_numbers()
function quick_page_numbers($first, $current, $total, $display, $link) {
	$page_settings = array (
		'prev_enabled'  => '<a href="{link}">&lt;&lt;<b>PREV</b></a>',
		'prev_disabled' => '&lt;&lt;<b>PREV</b>',
		'next_enabled'  => '<a href="{link}"><b>NEXT</b>&gt;&gt;</a>',
		'next_disabled' => '<b>NEXT</b>&gt;&gt;',
		'button_sep'    => ' &middot; ',
		'link_format'   => '<a href="{link}">[{page}]</a>',
		'active_format' => '<b>[{page}]</b>',
		'elipses'       => ' ... ',
		'page_sep'      => ' '
	);

	return full_page_numbers($first, $current, $total, $display, $link, $page_settings);
}

?>